/*
 * Generates a valid CSS selector using a given target node
 * @param {HTMLElement} target
 * @param {HTMLElement} [document.body] root
 * @param {boolean} [false] isAsync
 * @returns {Promise<string> | string}
 */
function cssSelectorGenerator(target, root, isAsync) {
    if (root === void 0) { root = document.body; }
    if (isAsync === void 0) { isAsync = false; }
    if (isAsync) {
        return new Promise(function (resolve, reject) {
            setTimeout(function () {
                try {
                    var result = cssSelectorHelper(target, root);
                    resolve(result);
                }
                catch (e) {
                    reject(e);
                }
            }, 10);
        });
    }
    return cssSelectorHelper(target, root);
}
/*
 * Traverses DOM tree starting from `target` to `root` and returns a valid path
 * @param {HTMLElement} target
 * @param {HTMLElement} root
 * @returns {Promise<string> | string}
 */
function cssSelectorHelper(target, root) {
    var output = [getReference(target)];
    var current = target;
    while (current !== root) {
        current = current === null || current === void 0 ? void 0 : current.parentElement;
        if (current) {
            var reference = getReference(current);
            output.push(reference);
        }
        else {
            throw new Error('root node can not be reached out! :/');
        }
    }
    return output.reverse().join(' > ');
}
/*
 * Returns the element id and in case it is not possible then returns its tag name
 * @param {HTMLElement} target
 * @returns {string}
 */
function getReference(element) {
    if (element.hasAttribute('id')) {
        var id = element.getAttribute('id');
        return "#".concat(id);
    }
    return element.tagName.toLowerCase();
}
export default cssSelectorGenerator;
//# sourceMappingURL=index.js.map